// Determines the default vertical amount of speed given to the player when it slips
// If zero, it gets defaulted to 15
local banana_vspeed = 15
// Determines the default horizontal amount of speed given to the player when it slips
// If zero, it will be determined by the player's speed
local banana_hspeed = 0

////////
////
//
//   THE "SOC" STUFF
//
////
////////

freeslot(
	"sfx_npslip",
	"sfx_nprspw",
	"SPR_BNNP",
	"S_BBP_BANANAPEEL",
	"S_BBP_BANANASPAWN",
	"S_BBP_BANANADIE",
	"MT_BBP_BANANAPEEL"
)

sfxinfo[sfx_npslip].caption = "Slip up"
sfxinfo[sfx_nprspw].caption = "Magical apparition"

states[S_BBP_BANANAPEEL] = {
	sprite = SPR_BNNP,
	frame = A,
	tics = -1,
	nextstate = S_BBP_BANANAPEEL
}
states[S_BBP_BANANASPAWN] = {
	sprite = SPR_BNNP,
	frame = B|FF_ANIMATE,
	var1 = 15,
	var2 = 3,
	tics = 45,
	nextstate = S_BBP_BANANAPEEL
}
states[S_BBP_BANANADIE] = {
	sprite = SPR_BNNP,
	frame = A,
	tics = 105,
	nextstate = S_NULL
}

mobjinfo[MT_BBP_BANANAPEEL] = {
	//$Name 'Nana Peel
	//$Category Baldi's Item Box
	//$Sprite BNNPA0
	//$Arg0 Vertical thrust
	//$Arg0Tooltip "The vertical amount of speed given to the player when it slips.\nIf zero, it switches to its default value."
	//$Arg1 Horizontal thrust
	//$Arg1Tooltip "The horizontal amount of speed given to the player when it slips.\nIf zero, it will be determined by the player's speed."
	//$Arg2 Respawn?
	//$Arg2Type 11
	//$Arg2Enum YesNo
	//$Arg2Tooltip "If 'Yes', this 'Nana Peel will appear again after 3 seconds from its latest use."
	doomednum = 4701,
	spawnstate = S_BBP_BANANAPEEL,
	deathstate = S_BBP_BANANADIE,
	radius = 30*FRACUNIT,
	height = 32*FRACUNIT,
	flags = MF_SOLID|MF_SLIDEME|MF_NOCLIPHEIGHT|MF_NOGRAVITY
}

////////
////
//
//   THE CODE
//
////
////////

// Don't move!
addHook("PlayerCmd", function(player, cmd)
	if player.banana_slipped
		cmd.forwardmove = 0
		cmd.sidemove = 0
	end
end)

// Reset these on spawn
addHook("PlayerSpawn", function(player)
	player.banana_slipped = false
	player.banana_sliptime = 0
	player.banana_sliproll = 0
	player.banana_sliprotation = 0
end)

// You slip, you fly.
addHook("PlayerThink", function(player)
	if player.mo
	and player.mo.valid
		if player.banana_slipped
			if player.banana_sliptime
				player.banana_sliptime = $+1
			else
				player.banana_sliptime = 1
			end

			if player.banana_sliptime > 1
				if player.banana_sliptime == 2
					if player.banana_sliprotation
					and player.banana_sliprotation > 0
						player.banana_sliprotation = -1
					else
						player.banana_sliprotation = 1
					end
				end
				if player.banana_sliproll
					player.banana_sliproll = $+((player.speed/FRACUNIT/2+1)*ANG1)*(-player.banana_sliprotation)
				else
					player.banana_sliproll = ((player.speed/FRACUNIT/2+1)*ANG1)*(-player.banana_sliprotation)
				end
				player.mo.rollangle = $+((player.speed/FRACUNIT/2+1)*ANG1)*(-player.banana_sliprotation)
				player.drawangle = $+((player.speed/FRACUNIT/2+1)*ANG1)*(-player.banana_sliprotation)
			end

			if player.banana_sliptime < 3
				player.mo.state = S_PLAY_RIDE
			else
				player.mo.state = S_PLAY_PAIN
			end

			if (player.mo.eflags & MFE_JUSTHITFLOOR
			or P_PlayerInPain(player)
			or player.banana_sliptime > 5*TICRATE)
			and player.banana_sliptime > 2
				if not (player.mo.eflags & MFE_JUSTHITFLOOR)
				and not P_PlayerInPain(player)
					player.mo.state = S_PLAY_FALL
				end

				player.banana_slipped = false
			elseif player.banana_sliptime > 2
				if not (player.charflags & SF_NOJUMPSPIN)
				or player.charability2 == CA2_SPINDASH
					if ((P_MobjFlip(player.mo) > 0
					and player.mo.z < player.banana_slipz)
					or (P_MobjFlip(player.mo) < 0
					and player.mo.z > player.banana_slipzflip))
					and player.cmd.buttons & BT_SPIN
						S_StartSound(player.mo, sfx_s1a2, player)
						player.mo.state = S_PLAY_ROLL
						player.pflags = $|PF_JUMPED & ~PF_THOKKED
						player.banana_slipped = false

						local ghost = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_THOK)
						ghost.tics = 6
						ghost.fuse = 6
						ghost.skin = player.mo.skin
						ghost.sprite = player.mo.sprite
						ghost.sprite2 = player.mo.sprite2
						ghost.frame = $|player.mo.frame
						ghost.color = player.mo.color
						ghost.colorized = true
						ghost.scale = player.mo.scale
						ghost.destscale = player.mo.scale*6
						ghost.dispoffset = -1
						ghost.target = player.mo
						ghost.banana_ghosttarget = player.mo
						local whiteflash = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, MT_THOK)
						whiteflash.skin = player.mo.skin
						whiteflash.sprite = player.mo.sprite
						whiteflash.sprite2 = player.mo.sprite2
						whiteflash.frame = $|player.mo.frame
						whiteflash.color = SKINCOLOR_SUPERSILVER2
						whiteflash.colorized = true
						whiteflash.scale = player.mo.scale
						whiteflash.dispoffset = 1
						whiteflash.renderflags = $|RF_FULLBRIGHT
						whiteflash.blendmode = AST_ADD
						whiteflash.target = player.mo
						whiteflash.banana_ghosttarget = player.mo
					end
				end
			end
		else
			if player.banana_sliproll
				player.mo.rollangle = 0
			end

			player.banana_sliproll = 0
			player.banana_sliptime = 0
		end
	end
end)

// Special effects for slip recovery
// Also saves a banana's spawn position
addHook("MobjThinker", function(mobj)
	if mobj
	and mobj.valid
		if mobj.banana_respawnpoint
			if mobj.tics == 1
				local banana = P_SpawnMobjFromMobj(mobj, 0, 0, 0, MT_BBP_BANANAPEEL)
				banana.banana_respawn = true
				banana.banana_once = true
				banana.banana_vspeed = mobj.banana_vspeed
				banana.banana_hspeed = mobj.banana_hspeed
				banana.state = S_BBP_BANANASPAWN
				banana.scale = mobj.scale
				if P_MobjFlip(mobj) < 0
					banana.eflags = $|MFE_VERTICALFLIP
				end

				S_StartSound(banana, sfx_nprspw)
			end
		elseif mobj.target
		and mobj.target.valid
		and mobj.banana_ghosttarget
		and mobj.banana_ghosttarget.valid
		and mobj.target == mobj.banana_ghosttarget
		and mobj.target.player
			A_CapeChase(mobj)

			local GetThokTranslucency = function(int)
				if int > 9
				or int < 0
					return 0
				else
					return FF_TRANS10*(8-(int-1))
				end
			end

			mobj.skin = mobj.target.skin
			mobj.sprite = mobj.target.sprite
			mobj.sprite2 = mobj.target.sprite2
			mobj.frame = mobj.target.frame|GetThokTranslucency(mobj.tics)
			mobj.angle = mobj.target.player.drawangle
			mobj.rollangle = mobj.target.rollangle
		end
	end
end, MT_THOK)

// Banana physics & speed arguments
addHook("MobjThinker", function(mobj)
	if mobj
	and mobj.valid
		if mobj.health
			if mobj.state == S_BBP_BANANAPEEL
				mobj.flags = $|MF_SOLID|MF_SLIDEME & ~MF_NOCLIP & ~MF_NOCLIPTHING
			else
				mobj.flags = $|MF_NOCLIP|MF_NOCLIPTHING & ~MF_SOLID & ~MF_SLIDEME
			end

			if not mobj.banana_once
				if mobj.spawnpoint
				and mobj.spawnpoint.args[0]
					mobj.banana_vspeed = abs(mobj.spawnpoint.args[0])
				elseif banana_vspeed
				and abs(banana_vspeed)
					mobj.banana_vspeed = abs(banana_vspeed)
				else
					mobj.banana_vspeed = 15
				end

				if mobj.spawnpoint
				and mobj.spawnpoint.args[1]
					mobj.banana_hspeed = abs(mobj.spawnpoint.args[1])
				elseif banana_hspeed
				and abs(banana_hspeed)
					mobj.banana_hspeed = abs(banana_hspeed)
				else
					mobj.banana_hspeed = 0
				end

				mobj.banana_once = true
			end
		else
			if not mobj.banana_hop
				if (mobj.spawnpoint
				and not mobj.spawnpoint.args[2])
				or mobj.banana_respawn
					local respawnpoint = P_SpawnMobjFromMobj(mobj, 0, 0, 0, MT_THOK)
					respawnpoint.tics = 105
					respawnpoint.fuse = 105
					respawnpoint.scale = mobj.scale
					respawnpoint.banana_vspeed = mobj.banana_vspeed
					respawnpoint.banana_hspeed = mobj.banana_hspeed
					respawnpoint.banana_respawnpoint = true
					respawnpoint.flags2 = $|MF2_DONTDRAW
					if P_MobjFlip(mobj) < 0
						respawnpoint.eflags = $|MFE_VERTICALFLIP
					end
				end

				mobj.momz = 10*mobj.scale*P_MobjFlip(mobj)
				mobj.banana_hop = true
			end

			mobj.momz = $-mobj.scale*P_MobjFlip(mobj)
			P_InstaThrust(mobj, mobj.banana_angle, mobj.banana_speed*mobj.scale*3/2)
		end
	end
end, MT_BBP_BANANAPEEL)

// Someone's going to slip!!
addHook("MobjCollide", function(mobj, mo)
	if mobj
	and mobj.valid
	and mobj.health
	and mo
	and mo.valid
	and mo.health
	and mo.player
	and mo.z <= mobj.z+mobj.height+FRACUNIT
	and mobj.z <= mo.z+mo.height+FRACUNIT
		local bananamomxy = mobj.banana_hspeed
		if not bananamomxy
			bananamomxy = mo.player.speed/FRACUNIT
		end

		mobj.banana_speed = bananamomxy
		mobj.banana_angle = R_PointToAngle2(mo.x, mo.y, mobj.x, mobj.y)

		mo.momz = mobj.banana_vspeed*mobj.scale
		P_InstaThrust(mo, mo.player.drawangle, bananamomxy*mobj.scale)

		S_StartSound(mo, sfx_npslip)
		mo.player.pflags = $ & ~PF_JUMPED & ~PF_SPINNING|PF_THOKKED
		mo.player.banana_sliptime = 0
		mo.player.banana_slipz = mobj.z
		mo.player.banana_slipzflip = mobj.z+mobj.height
		mo.player.banana_slipped = true

		P_KillMobj(mobj)

		return true
	end
end, MT_BBP_BANANAPEEL)